@@ -1,4 +1,5 @@
 import sys
+from pathlib import Path
 
 from vnpy.event import EventEngine
 from vnpy.trader.engine import MainEngine
@@ -11,10 +12,10 @@ from ..engine import APP_NAME, WebEngine
 class WebManager(QtWidgets.QWidget):
     """网页服务器管理界面"""
 
-    setting_filename = "web_trader_setting.json"
-    setting_filepath = get_file_path(setting_filename)
+    setting_filename: str = "web_trader_setting.json"
+    setting_filepath: Path = get_file_path(setting_filename)
 
-    def __init__(self, main_engine: MainEngine, event_engine: EventEngine):
+    def __init__(self, main_engine: MainEngine, event_engine: EventEngine) -> None:
         """"""
         super().__init__()
 
@@ -24,48 +25,54 @@ class WebManager(QtWidgets.QWidget):
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """初始化界面"""
         self.setWindowTitle("Web服务")
 
-        setting = load_json(self.setting_filepath)
-        username = setting.get("username", "vnpy")
-        password = setting.get("password", "vnpy")
-        req_address = setting.get("req_address", "tcp://127.0.0.1:2014")
-        sub_address = setting.get("sub_address", "tcp://127.0.0.1:4102")
-        host = setting.get("host", "127.0.0.1")
-        port = setting.get("port", "8000")
+        setting: dict = load_json(self.setting_filepath)
+        username: str = setting.get("username", "vnpy")
+        password: str = setting.get("password", "vnpy")
+        req_address: str = setting.get("req_address", "tcp://127.0.0.1:2014")
+        sub_address: str = setting.get("sub_address", "tcp://127.0.0.1:4102")
+        host: str = setting.get("host", "127.0.0.1")
+        port: str = setting.get("port", "8000")
 
-        self.username_line = QtWidgets.QLineEdit(username)
-        self.password_line = QtWidgets.QLineEdit(password)
-        self.req_line = QtWidgets.QLineEdit(req_address)
-        self.sub_line = QtWidgets.QLineEdit(sub_address)
-        self.host_line = QtWidgets.QLineEdit(host)
-        self.port_line = QtWidgets.QLineEdit(port)
+        self.username_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit(username)
+        self.password_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit(password)
+        self.req_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit(req_address)
+        self.sub_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit(sub_address)
+        self.host_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit(host)
+        self.port_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit(port)
 
-        self.start_button = QtWidgets.QPushButton("启动")
+        self.start_button: QtWidgets.QPushButton = QtWidgets.QPushButton("启动")
         self.start_button.clicked.connect(self.start)
 
-        self.text_edit = QtWidgets.QTextEdit()
+        self.end_button: QtWidgets.QPushButton = QtWidgets.QPushButton("停止")
+        self.end_button.clicked.connect(self.end)
+
+        self.text_edit: QtWidgets.QTextEdit = QtWidgets.QTextEdit()
         self.text_edit.setReadOnly(True)
 
-        form = QtWidgets.QFormLayout()
+        form: QtWidgets.QFormLayout = QtWidgets.QFormLayout()
         form.addRow("用户名", self.username_line)
         form.addRow("密码", self.password_line)
         form.addRow("请求地址", self.req_line)
-        form.addRow("请阅地址", self.sub_line)
+        form.addRow("订阅地址", self.sub_line)
         form.addRow("监听地址", self.host_line)
         form.addRow("监听端口", self.port_line)
         form.addRow(self.start_button)
+        form.addRow(self.end_button)
 
-        hbox = QtWidgets.QHBoxLayout()
+        self.end_button.setEnabled(False)
+
+        hbox: QtWidgets.QHBoxLayout = QtWidgets.QHBoxLayout()
         hbox.addLayout(form)
         hbox.addWidget(self.text_edit)
         self.setLayout(hbox)
 
         self.resize(1000, 500)
 
-    def start(self):
+    def start(self) -> None:
         """启动引擎"""
         username: str = self.username_line.text()
         password: str = self.password_line.text()
@@ -75,7 +82,7 @@ class WebManager(QtWidgets.QWidget):
         port: str = self.port_line.text()
 
         # 保存配置
-        setting = {
+        setting: dict = {
             "username": username,
             "password": password,
             "req_address": req_address,
@@ -90,7 +97,7 @@ class WebManager(QtWidgets.QWidget):
         self.start_button.setDisabled(True)
 
         # 初始化Web服务子进程
-        self.process = QtCore.QProcess(self)
+        self.process: QtCore.QProcess = QtCore.QProcess(self)
         self.process.setProcessChannelMode(self.process.MergedChannels)
 
         self.process.readyReadStandardOutput.connect(self.data_ready)
@@ -99,16 +106,19 @@ class WebManager(QtWidgets.QWidget):
         self.process.finished.connect(self.web_finished)
 
         # 启动子进程
-        cmd = [
+        cmd: list = [
             "-m"
             "uvicorn",
             "vnpy_webtrader.web:app",
             f"--host={host}",
-            f"--port={port}",
-            "--reload"
+            f"--port={port}"
         ]
         self.process.start(sys.executable, cmd)
 
+    def end(self) -> None:
+        """终止引擎"""
+        self.process.kill()
+
     def web_started(self) -> None:
         """Web进程启动"""
         self.text_edit.append("Web服务器启动")
@@ -124,17 +134,32 @@ class WebManager(QtWidgets.QWidget):
         ]:
             w.setEnabled(False)
 
+        self.end_button.setEnabled(True)
+
     def web_finished(self) -> None:
         """Web进程结束"""
         self.text_edit.append("Web服务器停止")
 
+        for w in [
+            self.username_line,
+            self.password_line,
+            self.req_line,
+            self.sub_line,
+            self.host_line,
+            self.port_line,
+            self.start_button
+        ]:
+            w.setEnabled(True)
+
+        self.end_button.setEnabled(False)
+
     def data_ready(self) -> None:
         """更新进程有数据可读"""
-        text = bytes(self.process.readAll())
+        text: bytes = bytes(self.process.readAll())
 
         try:
-            text = text.decode("UTF8")
+            text: str = text.decode("UTF8")
         except UnicodeDecodeError:
-            text = text.decode("GBK")
+            text: str = text.decode("GBK")
 
         self.text_edit.append(text)
